<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
	<head>
		<title>{{ sitetitle }}</title>
		<meta charset="utf-8" />
		<!-- <script src="/static/main.js"></script> -->
		<link href="static/css/main.css" type="text/css" rel="stylesheet">
	</head>

	<body>
		<div class="fullscreen">
			<div class="splash">
				<center>
					<h1>{{ sitetitle }}</h1>
					<h2>{{ translate('Congratulations your PySite is up and running') }}<h2>
				</center>
			</div>
		</div>
	</body>
</html>
