 
body {
	width: 100%;
	height: 100%;
}

div.fullscreen {
	width: 100%;
	height: 100%
}

div.splash {
	background-image: url(../images/splash-background.png);
	width: 800px;
	height: 501px;
	position: absolute;
	left: 50%;
	top: 50%;
	margin-left: -400px;
	margin-top: -250px;
	color: white;
}